#include <cassert>
#include "testlib.h"
#include "random"

using ll = long long;

using namespace std;

int get(int from, int to) {
    return rnd.next(from, to);
}

bool is_prime(ll x) {
    for (ll i = 2; i * i <= x; i++) {
        if (x % i == 0) {
            return false;
        }
    }
    return true;
}

static const ll max_val = 1e9;

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);

    vector<ll> primes = {7, 23, 89, 113, 523, 887, 1129, 1327, 9551, 15683, 19609, 31397, 155921, 360653, 370261,
                         492113, 1349533, 1357201, 2010733, 4652353, 17051707, 20831323, 47326693, 122164747, 189695659,
                         191912783, 387096133, 436273009, 1294268491, 1453168141};
    vector<ll> deltas = {4, 6, 8, 14, 18, 20, 22, 34, 36, 44, 52, 72, 86, 96, 112, 114, 118, 132, 148, 154, 180,
                         210, 220, 222, 234, 248, 250, 282, 288, 292};
    for (int i = 0; i < primes.size(); i++) {
        ll x = primes[i] + 1;
        while (!is_prime(x)) {
            x++;
        }
        assert(x - primes[i] == deltas[i]);
    }

    int prime_idx = has_opt("prime_idx") ? opt<int>("prime_idx") : -1;

    if (prime_idx != -1) {
        prime_idx--;
        ll l = primes[prime_idx];
        ll delta = deltas[prime_idx];
        ll r = l + delta;

        vector<vector<ll>> data;
        for (ll s = l + 1; s < r; s++) {
            for (int k = 1; k < delta; k++) {
                if ((r - s) % (k + 1) == 0) {
                    data.push_back({s, k});
                }
            }
        }

        cout << "10\n";
        for (int i = 0; i < 5; i++) {
            int idx = get(0, data.size() - 1);
            ll x = min(max_val, 1ll * get(1, data[idx][0] - 1));
            if (i < 2) {
                x = get(1, data[idx][1]);
            }
            ll y = data[idx][0] - x;
            if (y > max_val) {
                x += y - max_val;
                y = max_val;
            }
            cout << x << ' ' << y << ' ' << data[idx][1] << '\n';
        }

        for (int i = 0; i < 5; i++) {
            ll s = get(l + 1, r - 1);
            ll x = min(max_val, 1ll * get(1, s - 1));
            ll y = s - x;
            if (y > max_val) {
                x += y - max_val;
                y = max_val;
            }
            cout << x << ' ' << y << ' ' << get(1, delta) << '\n';
        }
    } else {
        cout << "10\n";
        int t = 0;
        while (t < 10) {
            ll x = get(1, max_val);
            ll y = get(1, max_val);
            ll k = get(1, max_val);
            if (!is_prime(x + y)) {
                t++;
                cout << x << ' ' << y << ' ' << k << '\n';
            }
        }
    }
}
