//
// Created by Daniil Oreshnikov on 01.11.2024.
//

#include "testlib.h"
#include <vector>
#include <set>
#include <unordered_set>

using namespace std;

vector<vector<int>> g;

void dfs(int v, unordered_set<int> const &blocked, vector<int> &vis, int c, int &limit) {
    if (limit <= 0) return;
    if (blocked.count(v) && vis[v] != c) return;
    vis[v] = c;
    limit--;
    for (int u: g[v]) {
        if (!vis[u]) {
            dfs(u, blocked, vis, c, limit);
        }
    }
}

int main(int argc, char *argv[]) {

    registerGen(argc, argv, 1);

    int n = opt<int>("n"), m = opt<int>("m");
    int max_comp = opt<int>("mc");
    int tree_w = opt<int>("tw");

    g.resize(n);
    vector<int> p(n);
    set<pair<int, int>> edges;

    for (int i = 1; i < n; i++) {
        if (tree_w > 0) {
            p[i] = rnd.wnext(0, min(tree_w, i - 1), -1);
        } else if (tree_w < 0) {
            p[i] = rnd.wnext(max(0, i + tree_w), i - 1, 1);
        } else {
            p[i] = rnd.next(0, i - 1);
        }
        g[i].push_back(p[i]);
        g[p[i]].push_back(i);
        edges.emplace(i, p[i]);
    }

    for (int i = n - 1; i < m; i++) {
        int u = 0, v = 0;
        while (u == v || edges.count({u, v})) {
            u = rnd.next(n);
            v = rnd.next(n);
            if (u < v) swap(u, v);
        }
        g[u].push_back(v);
        g[v].push_back(u);
        edges.emplace(u, v);
    }

    auto roots = rnd.distinct(3, 0, n - 1);
    unordered_set<int> blocked(roots.begin(), roots.end());

    println(n, m);

    vector<int> vis(n);
    for (int i = 0; i < 3; i++) {
        int mc = max_comp;
        vis[roots[i]] = i + 1;
        dfs(roots[i], blocked, vis, i + 1, mc);

        vector<int> portal;
        for (int v = 0; v < n; v++) {
            if (vis[v] == i + 1) {
                portal.push_back(v + 1);
            }
        }
        println(portal.size());
        println(portal.begin(), portal.end());
    }

    for (auto [u,v] : edges) {
        if (rnd.next(2)) swap(u, v);
        println(u + 1, v + 1);
    }

}