#include <iostream>
#include <ext/pb_ds/tree_policy.hpp>
#include <ext/pb_ds/assoc_container.hpp>
#include "testlib.h"

using namespace std;
using namespace __gnu_pbds;

typedef tree<
        pair<int, int>,
        null_type,
        less<pair<int, int>>,
        rb_tree_tag,
        tree_order_statistics_node_update> ordered_set;

int get(int from, int to) {
    return rnd.next(from, to);
}

void restore(ordered_set &st) {
    int sz = 0;
    for (auto [_, cnt]: st) {
        sz += cnt;
    }
    st.clear();
    if (sz > 0)
        st.insert({0, sz});
}

void push(ordered_set &st) {
    auto lst = st.rbegin();
    if (st.empty()) {
        st.insert({0, 1});
    } else {
        auto p = *lst;
        st.erase(lst);
        p.second++;
        st.insert(p);
    }
}

const int max_n = 1e5, max_m = max_n, max_k = max_n;

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);

    int n = has_opt("n") ? opt<int>("n") : max_n;
    int m = has_opt("m") ? opt<int>("m") : max_m;
    int k = has_opt("k") ? opt<int>("k") : max_k;

    int fast = has_opt("fast") ? opt<int>("fast") : 0;

    int cur = 1;
    cout << n << ' ' << m << '\n';

    for (int i = 1; i < n; i++) {
        cout << (cur++) << ' ';
    }
    cout << (cur++) << '\n';

    vector<string> reqs = {"push", "pop", "cut"};
    if (fast) {
//        for (int i = m - 1; i >= 1; i--) {
//            cout << "cut 1 " << i << '\n';
//        }
//        cout << "pop\n";
//        return 0;
        reqs = {"cut"};
    }

    ordered_set st;

    st.insert({0, n});

    for (int i = 1; i <= m; i++) {
        if (fast && i > m / 3 * 2) {
            reqs = {"push", "pop", "cut"};
        }
        if (i % k == 0) {
            cout << "restore\n";
            restore(st);
            continue;
        }
        string req = reqs[get(0, reqs.size() - 1)];

        if (req == "push") {
            cout << "push " << (cur++);
            push(st);
        } else if (req == "pop") {
            if (!st.empty()) {
                cout << "pop";
                ordered_set copy;
                for (auto [idx, cnt]: st) {
                    if (cnt > 1) {
                        copy.insert({idx + 1, cnt - 1});
                    }
                }
                swap(copy, st);
            } else {
                cout << "push " << (cur++);
                push(st);
            }
        } else if (req == "restore") {
            cout << "restore";
            restore(st);
        } else {
            vector<pair<int, int>> bebr;
            for (auto p: st) {
                if (p.second > 1) bebr.push_back(p);
            }
            if (bebr.empty()) {
                cout << "restore";
                restore(st);
            } else {
                cout << "cut ";
                pair<int, int> seg_to_del = bebr[get(0, bebr.size() - 1)];

                int w = st.order_of_key(seg_to_del) + 1;
                int count = get(1, seg_to_del.second - 1);

                st.erase(seg_to_del);
                st.insert({seg_to_del.first, count});
                st.insert({seg_to_del.first + count, seg_to_del.second - count});
                cout << w << ' ' << count;
            }
        }
        cout << '\n';
    }
}


//<#list 1..10 as i>
//gen -n 100 -m 100 ${i} > $
//        </#list>
//<#list 1..15 as i>
//gen ${i} > $
//</#list>
//<#list 1..15 as i>
//gen -n 50000 -m 55000 -fast 1 ${i} > $
//</#list>
//<#list 1..15 as i>
//gen -n 27000 -m 100000 -fast 1 ${i} > $
//</#list>
