#include <iostream>
#include <ext/pb_ds/tree_policy.hpp>
#include <ext/pb_ds/assoc_container.hpp>

using namespace std;
using namespace __gnu_pbds;

typedef tree<
        pair<int, int>,
        null_type,
        less<pair<int, int>>,
        rb_tree_tag,
        tree_order_statistics_node_update> ordered_set;

struct SegTree {
    vector<int> tree;
    int n;

    SegTree(const vector<int> &data) {
        n = (int) data.size();
        tree.resize(4 * n);
        build(1, 0, n, data);
    }

private:
    int recalc(int left, int right) {
        return left + right;
    }

    void build(int v, int l, int r, const vector<int> &data) {
        if (l + 1 == r) {
            tree[v] = data[l];
            return;
        }
        int m = (l + r) / 2;
        build(v * 2, l, m, data);
        build(v * 2 + 1, m, r, data);
        tree[v] = recalc(tree[v * 2], tree[v * 2 + 1]);
    }

    pair<int, int> get(int v, int l, int r, int lq, int rq, int k) {
        if (lq >= r || l >= rq) return {0, -1};
        if (lq <= l && r <= rq) {
            if (tree[v] < k) return {tree[v], -1};
            return {tree[v], get_bebra(v, l, r, k)};
        }
        int m = (l + r) / 2;
        auto idx = get(v * 2, l, m, lq, rq, k);
        if (idx.second != -1) return idx;
        auto g = get(v * 2 + 1, m, r, lq, rq, k - idx.first);
        g.first += idx.first;
        return g;
    }

    int get_bebra(int v, int l, int r, int k) {
        if (l + 1 == r) return l;
        int m = (l + r) / 2;
        int cnt_left = tree[v * 2]; // кво едениц слева
        if (cnt_left >= k) return get_bebra(v * 2, l, m, k);
        return get_bebra(v * 2 + 1, m, r, k - cnt_left);
    }

    void update(int v, int l, int r, int ind) {
        if (l + 1 == r) {
            tree[v] = 1 - tree[v];
            return;
        }
        int m = (l + r) / 2;
        if (ind < m) update(v * 2, l, m, ind);
        else update(v * 2 + 1, m, r, ind);
        tree[v] = recalc(tree[v * 2], tree[v * 2 + 1]);
    }

public:
    int get(int l, int r, int k) {
        return get(1, 0, n, l, r, k).second;
    }

    int get(int l, int k) {
        return get(l, n, k);
    }

    void upd(int ind) {
        update(1, 0, n, ind);
    }
};

const int max_n = 4e5 + 100;

int a[max_n];
int current = 0;

signed main() {
    int n, q;
    cin >> n >> q;

    for (int i = 0; i < n; i++) {
        cin >> a[i];
    }

    current = n;

    ordered_set st;
    st.insert({0, n});
    SegTree idxs(vector<int>(max_n, 1));

    int sz = n;

    while (q--) {
        string req;
        cin >> req;
        if (req == "push") {
            cin >> a[current++];
            auto it = st.rbegin();
            if (st.empty()) {
                st.insert({current - 1, 1});
            } else {
                auto cur = *it;
                st.erase(it);
                cur.second++;
                st.insert(cur);
            }
            sz++;
        } else if (req == "pop") {
            cout << a[st.begin()->first] << '\n';
            ordered_set copy;
            for (auto [idx, cnt]: st) {
                if (cnt != 1) {
                    copy.insert({idxs.get(idx + 1, 1), cnt - 1});
                }
                sz--;
                idxs.upd(idx);
            }
            swap(copy, st);
        } else if (req == "cut") {
            int i, j;
            cin >> i >> j;
            auto it = st.find_by_order(i - 1);
            auto cur = *it;
            st.erase(it);
            st.insert({cur.first, j});
            st.insert({idxs.get(cur.first, j + 1), cur.second - j});
        } else if (req == "restore") {
            auto start = st.begin()->first;
            st.clear();
            if (sz > 0) st.insert({start, sz});
        } else {
            assert(false);
        }
    }
}
