/*
Author: @doreshnikov
Created on: 03.11.2024/01:47
*/

#include <iostream>
#include <random>
#include <chrono>
#include <set>

using namespace std;

typedef unsigned long long ull;
typedef unsigned int uint;

void mcin() {}

template<typename T, typename ...Tail>
void mcin(T &&arg, Tail &&...args) {
    cin >> arg;
    mcin(args...);
}

#define in(type, ...) type __VA_ARGS__; mcin(__VA_ARGS__);

//int seed = chrono::high_resolution_clock::now().time_since_epoch().count();
int seed = 3;
mt19937 rnd(seed);

template<typename T>
struct node;

template<typename T>
struct node {
    typedef node *nodeptr;

    int size, y;
    T value;
    nodeptr left = nullptr;
    nodeptr right = nullptr;

    explicit node(T const &value) : value(std::move(value)), y(rnd()), size(1) {}
};

template<typename T>
bool operator<(typename node<T>::nodeptr const &n1, typename node<T>::nodeptr const &n2) {
    return n1->value < n2->value;
}

template<typename T>
int _size(node<T> *root) {
    return root ? root->size : 0;
}

template<typename T>
node<T> *recalc(node<T> *root) {
    root->size = 1 + _size(root->left) + _size(root->right);
    return root;
}

template<typename T>
node<T> *merge(node<T> *left, node<T> *right) {
    if (!left) return right;
    if (!right) return left;
    if (left->y > right->y) {
        left->right = merge(left->right, right);
        return recalc(left);
    } else {
        right->left = merge(left, right->left);
        return recalc(right);
    }
}

template<typename T>
pair<node<T> *, node<T> *> split(node<T> *root, int k) {
    if (!root) return {nullptr, nullptr};
    if (_size(root->left) >= k) {
        auto [ll, lr] = split(root->left, k);
        root->left = lr;
        return {ll, recalc(root)};
    } else {
        auto [rl, rr] = split(root->right, k - _size(root->left) - 1);
        root->right = rl;
        return {recalc(root), rr};
    }
}

template<typename T>
node<T> *front(node<T> *root) {
    if (!root) return nullptr;
    if (!root->left) return root;
    return front(root->left);
}

template<typename T>
node<T> *back(node<T> *root) {
    if (!root) return nullptr;
    if (!root->right) return root;
    return back(root->right);
}

struct queue {
    node<int> *elements;

    int size() const {
        return _size(elements);
    }

    int pop() {
        auto [l, r] = split(elements, 1);
        elements = r;
        return l->value;
    }

    void push(int value) {
        elements = merge(elements, new node(value));
    }
};

node<queue> *pop_from_all(node<queue> *root) {
    if (!root) return nullptr;
    auto left = pop_from_all(root->left);
    auto right = pop_from_all(root->right);

    root->value.pop();
    root = new node<queue>(root->value);
    if (root->value.size() == 0) {
        root = nullptr;
    }

    return merge(left, merge(root, right));
}

void restore(node<queue> *root, node<int> *&target) {
    if (!root) return;
    restore(root->left, target);
    target = merge(target, root->value.elements);
    restore(root->right, target);
}

int main() {

    in(int, n, q)
    auto *root = new node<queue>(queue{nullptr});

    for (int i = 0; i < n; i++) {
        in(int, ai);
        root->value.elements = merge(root->value.elements, new node<int>(ai));
    }

    for (int i = 0; i < q; i++) {
        in(string, cmd)
        if (cmd == "push") {
            in(int, x)
            if (!root) root = new node<queue>(queue{nullptr});
            back(root)->value.push(x);
        } else if (cmd == "pop") {
            auto first = front(root);
            cout << front(first->value.elements)->value << '\n';
            root = pop_from_all(root);
        } else if (cmd == "cut") {
            in(int, x, y)
            auto [m, r] = split(root, x);
            auto [l, elem] = split(m, _size(m) - 1);

            auto [il, ir] = split(elem->value.elements, y);
            root = merge(
                    merge(l, new node<queue>(queue{il})),
                    merge(new node<queue>(queue{ir}), r)
            );
        } else {
            node<int> *target = nullptr;
            restore(root, target);
            root = new node<queue>(queue{target});
        }
    }

}