#include <vector>
#include <unordered_set>
#include "testlib.h"

using std::vector;

int n;
vector<vector<int>> g;
vector<int> colors;

void validate_set(int v, vector<bool> &may_visit) {
    ensuref(may_visit[v], "Panic! visiting unwanted node %d", v);
    may_visit[v] = false;
    for (int u : g[v]) {
        if (may_visit[u]) {
            validate_set(u, may_visit);
        }
    }
}

int check_split(InStream &stream) {
    int s = stream.readInt(1, n, "s");
    vector<bool> may_visit(n, false);
    vector<bool> covered(n, false);
    for (int i = 0; i < s; i++) {
        std::unordered_set<int> used_colors;
        int cnt_size = stream.readInt(1, n, "n_i");
        auto cnt_set = stream.readInts(cnt_size, 1, n);

        for (auto v : cnt_set) {
            used_colors.insert(colors[v - 1]);
            stream.ensuref(used_colors.size() <= 2, "More than two different numbers in set %d!", i + 1);
            may_visit[v - 1] = true;
            covered[v - 1] = true;
        }
        validate_set(cnt_set.front() - 1, may_visit);

        for (auto v : cnt_set) {
            stream.ensuref(!may_visit[v - 1],
                           "set %d is not connected. %d is not reachable from %d!",
                           i + 1, v, cnt_set.front());
        }
    }
    for (int i = 0; i < n; i++) {
        stream.ensuref(covered[i], "node %d is not covered!", i + 1);
    }
    return s;
}

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);

    n = inf.readInt();
    colors = inf.readInts(n);
    g.resize(n);
    for (int i = 1; i < n; i++) {
        int v = inf.readInt() - 1;
        int u = inf.readInt() - 1;
        g[v].push_back(u);
        g[u].push_back(v);
    }
    int jury_ans = check_split(ans);
    int participant_ans = check_split(ouf);
    if (jury_ans < participant_ans) {
        quitf(_wa,
              "jury has the better answer: jans = %d, pans = %d\n",
              jury_ans, participant_ans);
    } else if (jury_ans == participant_ans) {
        quitf(_ok, "answer = %d", jury_ans);
    } else {
        quitf(_fail, "participant found better answer: %d vs %d", participant_ans, jury_ans);
    }
}